(require "str")
(require "drakma")
(require "cl-store")
(require "local-time")

(defvar all_dbs (make-hash-table :test (quote equal)))

(defun reset-costs (costs)
  (setf (gethash "misc" costs) 0)
  "Values reset")

(if (probe-file "costs.obj")
    (progn
      (print "restoring costs...")
      (setf all_dbs (cl-store:restore "costs.obj"))))

(defun add-number (num)
      (if (not (gethash num all_dbs))
	  (progn
	    (print (format nil "Initializing DB for ~a" num))
	    (setf (gethash num all_dbs)
		  (make-hash-table :test (quote equal)))
	    (reset-costs (gethash num all_dbs)))
	(print (format nil "Entry already found for ~a" num)))
      "Number added. Welcome.")

(defun allitems (costs)
  (with-output-to-string (s)
    (maphash
     (function (lambda (key val)
		 (format s "~a: $~a~%" key val)))
     costs)))

(defun sendsms (number msg)
  (drakma:http-request
   (format nil "https://api.twilio.com/2010-04-01/Accounts/~a/Messages.json" sid)
   :method :post
   :parameters (list (cons "To"  number)
		     (cons "From"  tracknumber)
		     (cons "Body"  msg))
   :basic-authorization (list sid token)))

(defun str-to-float (data)
  (with-input-from-string (in data) (read in)))

(defun report-balance (cmd costs)
  (let ((bal (gethash cmd costs)))
    (if bal
	(format nil "~a balance is $~a." cmd bal)
      "Invalid command")))

(defun update-balance (cmd arg note costs from)
  (let ((amt (str-to-float arg))
	(ctime (local-time:format-timestring
		nil
		(local-time:now)
		:format local-time:+rfc-1123-format+)))
    (if (gethash cmd costs)
	(prog1
	    (format nil "~a balance is now $~a" cmd
		    (decf (gethash cmd costs) amt))
	  (cl-store:store all_dbs "costs.obj")
	  (with-open-file (s (format nil "log-~a.txt" from)
			     :direction :output
			     :if-exists :append
			     :if-does-not-exist :create)
			  (format s "~a~C~a~C~a~C~a~%"
				  ctime #\tab cmd #\tab arg #\tab note)))
      "Invalid command")))

(defun setamt (cmd arg costs)
  (let ((amt (str-to-float arg)))
    (if (gethash cmd costs)
	(prog1
	    (format nil "~a balance is now $~a" cmd
		    (setf (gethash cmd costs) amt)))
      "Invalid command")))

(defun process (from body costs)
  (destructuring-bind
   (arg &rest note)
   (str:split " " body)
   (cond
    ((equal arg "quotas") ;; they want to see all quotas
     (sendsms from (allitems costs)))
    ((equal arg "reset")
     (sendsms from (reset-costs costs)))
    ((equal arg "set")
     (sendsms from (setamt "misc" (first note) costs)))
    (t ;; else - we have both a command and an arg so assume they're updating a balance
     (sendsms from (update-balance "misc" arg note costs from))))))

(defun sms-dispatch (req)   ;; This function is called by the server when a new HTTP request comes in.
  (let ((from (post-parameter "From" req))
	(body (post-parameter "Body" req)))
    (if (gethash from all_dbs)
	(map nil (lambda (onebody)
		   (process from onebody (gethash from all_dbs)))
	     (str:split #\Newline body))
      (sendsms from (add-number from))))
  "OK")
